/*
 * Copyright (C) 2009 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/*
 学習リモコン用に改造 SYSYTEC （京）
 */
package jp.systec.android.RemoteControl;

import jp.systec.android.RemoteControl.R;

import android.app.Activity;
import android.app.AlertDialog;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.Context;
import android.graphics.Color;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.os.Environment;
import android.util.Log;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.Window;
import android.view.View.OnClickListener;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ImageButton;
import android.widget.ListView;
import android.widget.TableLayout;
import android.widget.TextView;
import android.widget.Toast;
import android.os.Build;
import java.io.FileInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Arrays;
import java.util.Set;
import java.util.Timer;
import java.util.TimerTask;
import java.util.Vector;

/**
 * This is the main Activity that displays the current chat session.
 */
public class RemoteControl extends Activity {
    // Debugging
    private static final String TAG = "RemoteControl";
    private static final boolean D = true;     // デバッグ用
    private static final boolean D2 = false;   // デバッグ用
    // アプリ動作制御
    private static final boolean HANDLED_BINARY = true; // バイナリーデータを扱う場合 true(通常true)
    private static final boolean CMD_ON_FINIFH = false; // プログラム終了時に"E"コマンド送信する場合には true(通常 false)
    private static final String REMOTE_DEVICE_NAME = "SMART REMOCON"; // リモコンのデバイス名変更した場合にはここを変えます

    // Message types sent from the BluetoothChatService Handler
    public static final int MESSAGE_STATE_CHANGE = 1;
    public static final int MESSAGE_READ = 2;
    public static final int MESSAGE_WRITE = 3;
    public static final int MESSAGE_DEVICE_NAME = 4;
    public static final int MESSAGE_TOAST = 5;

    // Key names received from the BluetoothChatService Handler
    public static final String DEVICE_NAME = "device_name";
    public static final String TOAST = "toast";

    // Intent request codes
    private static final int REQUEST_CONNECT_DEVICE_SECURE = 1;
    private static final int REQUEST_CONNECT_DEVICE_INSECURE = 2;
    private static final int REQUEST_ENABLE_BT = 3;

    // リモコンデータ内部ストレージ保存ファイル名 SYSTEC
    private static final String FILE_NAME = "data_";
    // リモコンデータ外部ストレージ保存ファイル名 SYSTEC
    private static final String FILE_NAME_STRAGE = "data2_";
    // リモコン受信コマンド(R)  SYSTEC
    private static final String RCVE_CMD = "R";
    // リモコン送信コマンド(T)  SYSTEC
    private static final String SEND_CMD = "T";
    // リモコン送信コマンド(E)  SYSTEC
    private static final String FINISH_CMD = "E";
    // バッファ容量 SYSTEC
    private static final int BUFFER_SIZE = 1024;
    // 内部ストレージ／外部ストレージ切り替えフラグ(0=内部,1=外部)
    // 外部ストレージがあるときには自動的に1になる SYSTEC
    private static int mExternalMedia = 0;
    // 赤外線学習、赤外線送信を示すフラグ(赤外線学習の場合=1,赤外線送信の場合=2,それ以外=0)
    private static int mIRSendFlag = 0;
    // リモコン1コマンドあたりのバイト数
    private static final int COMMAND_SIZE = 160;
    // クリック直後の無効時間(ミリ秒)
    private static final int INHIBIT_TIME = 700;
    // データウィンドウ表示
    private static final boolean DWIN = false;   // データウィンドウにデータを表示しない場合はfalse


    // Layout Views
    private TextView mTitle;
    private ListView mConversationView;

    private Button mButton1 = null;// ボタン1
    private Button mButton2 = null;// ボタン2
    private Button mButton3 = null;// ボタン3
    //private Button mButton4 = null;// ボタン4
    //private Button mButton5 = null;// ボタン5
    private Button mButton6 = null;// ボタン6
    //private Button mButton7 = null;// ボタン7
    private Button mButton8 = null;// ボタン8
    private ImageButton mButton9 = null;// ボタン9
    private ImageButton mButton10 = null;// ボタン10

    private ImageButton mButton11 = null;// ボタン11
    private ImageButton mButton12 = null;// ボタン12
    private Button mButton13 = null;// ボタン13
    private Button mButton14 = null;// ボタン14
    private Button mButton15 = null;// ボタン15
    private Button mButton16 = null;// ボタン16
    private Button mButton17=null;// ボタン17(音声＋)
    //private Button mButton18 = null;// ボタン18
    private ImageButton mButton19 = null;// ボタン19
    //private Button mButton20 = null;// ボタン20

    private ImageButton mButton21 = null;// ボタン21
    private ImageButton mButton22 = null;// ボタン22
    private ImageButton mButton23 = null;// ボタン23
    private ImageButton mButton24 = null;// ボタン24
    private Button mButton25 = null;// ボタン25
    private Button mButton26 = null;// ボタン26
    private Button mButton27 = null;// ボタン27
    private Button mButton28 = null;// ボタン28
    private Button mButton29=null;// ボタン29(音声－)
    private ImageButton mButton30 = null;// ボタン30

    private Button mButton31 = null;// ボタン31
    private ImageButton mButton32 = null;// ボタン32
    private ImageButton mButton33 = null;// ボタン33
    private ImageButton mButton34 = null;// ボタン34
    //private Button mButton35 = null;// ボタン35
    private ImageButton mButton36 = null;// ボタン36
    private Button mButton37 = null;// ボタン37
    private Button mButton38 = null;// ボタン38
    private Button mButton39 = null;// ボタン39
    //private Button mButton40 = null;// ボタン40

    //private Button mButton41 = null;// ボタン41
    //private Button mButton42 = null;// ボタン42
    private ImageButton mButton43 = null;// ボタン43
    //private Button mButton44 = null;// ボタン44
    private ImageButton mButton45 = null;// ボタン45
    private ImageButton mButton46 = null;// ボタン46
    //private Button mButton47 = null;// ボタン47
    private ImageButton mButton48 = null;// ボタン48
    private Button mButton49 = null;// ボタン49
    private Button mButton50 = null;// ボタン50

    private Button mButton51 = null;// ボタン51
    private Button mButton52 = null;// ボタン52
    private Button mButton53 = null;// ボタン53
    private Button mButton54 = null;// ボタン54
    //private Button mButton55 = null;// ボタン55
    private Button mButton56 = null;// ボタン56
    private Button mButton57 = null;// ボタン57
    private Button mButton58 = null;// ボタン58
    private Button mButton59 = null;// ボタン59
    private ImageButton mButton60 = null;// ボタン60(電源)



    // Name of the connected device
    private String mConnectedDeviceName = null;
    // Array adapter for the conversation thread
    private ArrayAdapter<String> mConversationArrayAdapter;
    // String buffer for outgoing messages
    private StringBuffer mOutStringBuffer;
    // Local Bluetooth adapter
    private BluetoothAdapter mBluetoothAdapter = null;
    // Member object for the chat services
    private RemoteControlService mChatService = null;

    // モード
    private int mModeFlag = 0;//0:通常モード 1:学習モード

    // ボタンNo.
    private String mButtonNo = "1";//操作したボタンの番号

    byte[]  mCommandBuffer;   // 160バイト保存用
    // Member fields
    private Vector<BluetoothDevice> mDevices;
    private String mMainConnectDeviceName = null;
    private String mMainConnectDeviceAddress = null;
    // External StrageDirectory保持
    private String mExternalDirectory = null;
    // Return Intent extra
    public static String EXTRA_DEVICE_ADDRESS = "device_address";

    // タイマーイベントクラス
    class ButtonTimerTask extends TimerTask{

    	@Override
    	public void run() {
    		// mHandler2を通じてボタン操作を有効化
    		mHandler2.post( new Runnable() {
    			public void run() {
    				// タイマーを無効にする
    				if(mTimer != null){
    					mTimer.cancel();
    					mTimer = null;
    				}
    				// 該当ボタンを有効化(各ボタンイベントにおいて無効化処理を行った場合にはここで有効化を行う)
    				switch(Integer.parseInt(mButtonNo)){
    				case 1: //
        				if(mButton1 != null){
        					mButton1.setEnabled(true);
        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton1 is Enabled.");}
        				}
    					break;
    				case 2: //
        				if(mButton2 != null){
        					mButton2.setEnabled(true);
        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton2 is Enabled.");}
        				}
    					break;
    				case 3: //
        				if(mButton3 != null){
        					mButton3.setEnabled(true);
        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton3 is Enabled.");}
        				}
    					break;
//    				case 4: //
//        				if(mButton4 != null){
//        					mButton4.setEnabled(true);
//        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton4 is Enabled.");}
//        				}
//    					break;
//    				case 5: //
//        				if(mButton5 != null){
//        					mButton5.setEnabled(true);
//        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton5 is Enabled.");}
//        				}
//    					break;
    				case 6: //
        				if(mButton6 != null){
        					mButton6.setEnabled(true);
        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton6 is Enabled.");}
        				}
    					break;
//   				case 7: //
//        				if(mButton7 != null){
//        					mButton7.setEnabled(true);
//        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton7 is Enabled.");}
//        				}
//    					break;
    				case 8: //
        				if(mButton8 != null){
        					mButton8.setEnabled(true);
        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton8 is Enabled.");}
        				}
    					break;
    				case 9: //
        				if(mButton9 != null){
        					mButton9.setEnabled(true);
        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton9 is Enabled.");}
        				}
    					break;
    				case 10: //
        				if(mButton10 != null){
        					mButton10.setEnabled(true);
        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton10 is Enabled.");}
        				}
    					break;
    				case 11: //
        				if(mButton11 != null){
        					mButton11.setEnabled(true);
        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton11 is Enabled.");}
        				}
    					break;
    				case 12: //
        				if(mButton12 != null){
        					mButton12.setEnabled(true);
        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton12 is Enabled.");}
        				}
    					break;
    				case 13: //
        				if(mButton13 != null){
        					mButton13.setEnabled(true);
        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton13 is Enabled.");}
        				}
    					break;
    				case 14: //
        				if(mButton14 != null){
        					mButton14.setEnabled(true);
        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton14 is Enabled.");}
        				}
    					break;
    				case 15: //
        				if(mButton15 != null){
        					mButton15.setEnabled(true);
        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton15 is Enabled.");}
        				}
    					break;
    				case 16: // CH+
        				if(mButton16 != null){
        					mButton16.setEnabled(true);
        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton16 is Enabled.");}
        				}
    					break;
    				case 17: // 音声＋
        				if(mButton17 != null){
        					mButton17.setEnabled(true);
        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton17 is Enabled.");}
        				}
    					break;
//        				case 18: //
//        				if(mButton18 != null){
//        					mButton18.setEnabled(true);
//        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton18 is Enabled.");}
//        				}
//    					break;
    				case 19: //
        				if(mButton19 != null){
        					mButton19.setEnabled(true);
        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton19 is Enabled.");}
        				}
    					break;
//        				case 20: //
//        				if(mButton20 != null){
//        					mButton20.setEnabled(true);
//        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton20 is Enabled.");}
//        				}
//    					break;
    				case 21: //
        				if(mButton21 != null){
        					mButton21.setEnabled(true);
        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton21 is Enabled.");}
        				}
    					break;
    				case 22: //
        				if(mButton22 != null){
        					mButton22.setEnabled(true);
        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton22 is Enabled.");}
        				}
    					break;
    				case 23: //
        				if(mButton23 != null){
        					mButton23.setEnabled(true);
        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton23 is Enabled.");}
        				}
    					break;
    				case 24: //
        				if(mButton24 != null){
        					mButton24.setEnabled(true);
        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton24 is Enabled.");}
        				}
    					break;
    				case 25: //
        				if(mButton25 != null){
        					mButton25.setEnabled(true);
        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton25 is Enabled.");}
        				}
    					break;
    				case 26: //
        				if(mButton26 != null){
        					mButton26.setEnabled(true);
        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton26 is Enabled.");}
        				}
    					break;
    				case 27: //
        				if(mButton27 != null){
        					mButton27.setEnabled(true);
        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton27 is Enabled.");}
        				}
    					break;
    				case 28: // CH-
        				if(mButton28 != null){
        					mButton28.setEnabled(true);
        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton28 is Enabled.");}
        				}
        				break;
    				case 29: // 音声-
        				if(mButton29 != null){
        					mButton29.setEnabled(true);
        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton29 is Enabled.");}
        				}
        				break;
    				case 30: //
        				if(mButton30 != null){
        					mButton30.setEnabled(true);
        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton30 is Enabled.");}
        				}
    					break;
    				case 31: //
        				if(mButton31 != null){
        					mButton31.setEnabled(true);
        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton31 is Enabled.");}
        				}
    					break;
    				case 32: //
        				if(mButton32 != null){
        					mButton32.setEnabled(true);
        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton32 is Enabled.");}
        				}
    					break;
    				case 33: //
        				if(mButton33 != null){
        					mButton33.setEnabled(true);
        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton33 is Enabled.");}
        				}
    					break;
    				case 34: //
        				if(mButton34 != null){
        					mButton34.setEnabled(true);
        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton34 is Enabled.");}
        				}
    					break;
//        				case 35: //
//        				if(mButton35 != null){
//        					mButton35.setEnabled(true);
//        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton35 is Enabled.");}
//        				}
//    					break;
    				case 36: //
        				if(mButton36 != null){
        					mButton36.setEnabled(true);
        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton36 is Enabled.");}
        				}
    					break;
    				case 37: //
        				if(mButton37 != null){
        					mButton37.setEnabled(true);
        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton37 is Enabled.");}
        				}
    					break;
    				case 38: //
        				if(mButton38 != null){
        					mButton38.setEnabled(true);
        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton38 is Enabled.");}
        				}
    					break;
    				case 39: //
        				if(mButton39 != null){
        					mButton39.setEnabled(true);
        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton39 is Enabled.");}
        				}
    					break;
//        				case 40: //
//        				if(mButton40 != null){
//        					mButton40.setEnabled(true);
//        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton40 is Enabled.");}
//        				}
//    					break;
//        				case 41: //
//        				if(mButton41 != null){
//        					mButton41.setEnabled(true);
//        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton41 is Enabled.");}
//        				}
//    					break;
//        				case 42: //
//        				if(mButton42 != null){
//        					mButton42.setEnabled(true);
//        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton42 is Enabled.");}
//        				}
//    					break;
    				case 43: //
        				if(mButton43 != null){
        					mButton43.setEnabled(true);
        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton43 is Enabled.");}
        				}
    					break;
//        				case 44: //
//        				if(mButton44 != null){
//        					mButton44.setEnabled(true);
//        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton44 is Enabled.");}
//        				}
//    					break;
    				case 45: //
        				if(mButton45 != null){
        					mButton45.setEnabled(true);
        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton45 is Enabled.");}
        				}
    					break;
    				case 46: //
        				if(mButton46 != null){
        					mButton46.setEnabled(true);
        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton46 is Enabled.");}
        				}
    					break;
//        				case 47: //
//        				if(mButton47 != null){
//        					mButton47.setEnabled(true);
//        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton47 is Enabled.");}
//        				}
//    					break;
    				case 48: //
        				if(mButton48 != null){
        					mButton48.setEnabled(true);
        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton48 is Enabled.");}
        				}
    					break;
    				case 49: //
        				if(mButton49 != null){
        					mButton49.setEnabled(true);
        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton49 is Enabled.");}
        				}
    					break;
    				case 50: //
        				if(mButton50 != null){
        					mButton50.setEnabled(true);
        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton50 is Enabled.");}
        				}
    					break;
    				case 51: //
        				if(mButton51 != null){
        					mButton51.setEnabled(true);
        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton51 is Enabled.");}
        				}
    					break;
    				case 52: //
        				if(mButton52 != null){
        					mButton52.setEnabled(true);
        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton52 is Enabled.");}
        				}
    					break;
    				case 53: //
        				if(mButton53 != null){
        					mButton53.setEnabled(true);
        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton53 is Enabled.");}
        				}
    					break;
    				case 54: //
        				if(mButton54 != null){
        					mButton54.setEnabled(true);
        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton54 is Enabled.");}
        				}
    					break;
//    				case 55: //
//        				if(mButton55 != null){
//        					mButton55.setEnabled(true);
//        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton55 is Enabled.");}
//        				}
//    					break;
    				case 56: //
        				if(mButton56 != null){
        					mButton56.setEnabled(true);
        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton56 is Enabled.");}
        				}
    					break;
    				case 57: //
        				if(mButton57 != null){
        					mButton57.setEnabled(true);
        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton57 is Enabled.");}
        				}
    					break;
    				case 58: //
        				if(mButton58 != null){
        					mButton58.setEnabled(true);
        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton58 is Enabled.");}
        				}
    					break;
    				case 59: //
        				if(mButton59 != null){
        					mButton59.setEnabled(true);
        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton59 is Enabled.");}
        				}
    					break;
    				case 60: //
        				if(mButton60 != null){
        					mButton60.setEnabled(true);
        	                if(D){Log.d(TAG, "ButtonTimerTask() --- mButton60 is Enabled.");}
        				}
    					break;
        			default:
        				break;
    				}
    			}
    		});
    	}
    }

    // タイマー
    private Timer   mTimer = null;
    private ButtonTimerTask     timerTask = null;		//onClickメソッドでインスタンス生成

	private final Handler mHandler2 = new Handler();	//タイマーイベントハンドラー

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        if(D) Log.e(TAG, "+++ ON CREATE +++");

        // Set up the window layout
        requestWindowFeature(Window.FEATURE_CUSTOM_TITLE);
        setContentView(R.layout.main);
        getWindow().setFeatureInt(Window.FEATURE_CUSTOM_TITLE, R.layout.custom_title);

        // External Media Check
        mExternalMedia = (isExternalStorageAvailable()) ? 1 : 0;
        // Set up the custom title
        mTitle = (TextView) findViewById(R.id.title_left_text);
        mTitle.setText(R.string.app_name);
        if(mExternalMedia == 1){
        	mTitle.append("(*)");
        }
        mTitle = (TextView) findViewById(R.id.title_right_text);

        // Get local Bluetooth adapter
        mBluetoothAdapter = BluetoothAdapter.getDefaultAdapter();

        // If the adapter is null, then Bluetooth is not supported
        if (mBluetoothAdapter == null) {
            Toast.makeText(this, "Bluetooth is not available", Toast.LENGTH_LONG).show();
            finish();
            return;
        }
        // コマンドバッファー確保
        mCommandBuffer=new byte[COMMAND_SIZE];

        if(mExternalMedia == 1){
        	mExternalDirectory = this.getExternalFilesDir(null).toString();
        	if(D) Log.e(TAG, "ON Create(): mExternalSDCardDirectory=["+mExternalDirectory+"]");
        }
    }

    @Override
    public void onStart() {
        super.onStart();
        if(D) Log.e(TAG, "++ ON START ++");

        // If BT is not on, request that it be enabled.
        // setupChat() will then be called during onActivityResult
        if (!mBluetoothAdapter.isEnabled()) {
            Intent enableIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
            startActivityForResult(enableIntent, REQUEST_ENABLE_BT);
        // Otherwise, setup the chat session
        } else {
            if (mChatService == null) setupChat();
        }
        // 既に登録済みのデバイスを検索
        // 発見したデバイスを格納するVectorを定義
        mDevices = new Vector<BluetoothDevice>();
        // Bluetoothデバイス一覧の取得
        Set<BluetoothDevice> devices = mBluetoothAdapter.getBondedDevices();

        for (BluetoothDevice device : devices) {
        	mDevices.add(device);
        	if(device.getName().equalsIgnoreCase(REMOTE_DEVICE_NAME)){
        		mMainConnectDeviceName = device.getName();
        		mMainConnectDeviceAddress = device.getAddress();
                // Toast.makeText(this, "Device Found!!: Device Name=[" + device.getName() + "], Address=[" + device.getAddress() + "]", Toast.LENGTH_LONG).show();
                if(D){Log.d(TAG, "onCreate()-Find TargetDevice Name=["+mMainConnectDeviceName+"]"+",Address=["+mMainConnectDeviceAddress+"]");}
        		break;
        	}
        }

        if(mMainConnectDeviceAddress != null){
        	if(!mMainConnectDeviceAddress.isEmpty()){
        		// 見つかった
        		showIndicadeDialog(this, "", "ペアリング情報が見つかりました。\nアドレス=["+mMainConnectDeviceAddress+"]\nのスマートリモコンに接続します。");
        	}
        }else{
        	showMsgDialog(this, "", "登録済みのリモコンが見つかりませんでした\nペアリングを実施してください。");
        }
    }

    @Override
    public synchronized void onResume() {
        super.onResume();
        if(D) Log.e(TAG, "+ ON RESUME +");

        if (mChatService != null) {
            // Only if the state is STATE_NONE, do we know that we haven't started already
            if (mChatService.getState() == RemoteControlService.STATE_NONE) {
              // Start the Bluetooth chat services
              mChatService.start();
            }
        }
    }

    private void setupChat() {
        Log.d(TAG, "setupChat()");

        // Initialize the array adapter for the conversation thread
        mConversationArrayAdapter = new ArrayAdapter<String>(this, R.layout.message);
        mConversationView = (ListView) findViewById(R.id.in);
        mConversationView.setAdapter(mConversationArrayAdapter);

        // ボタン1イベント
        mButton1 = (Button) findViewById(R.id.button_1);
        mButton1.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                if(mTimer != null) return;
            	mButtonNo = "1";
            	button_event(mButton1);
            }
        });

        // ボタン2イベント
        mButton2 = (Button) findViewById(R.id.button_2);
        mButton2.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                if(mTimer != null) return;
            	mButtonNo = "2";
            	button_event(mButton2);
            }
        });

        // ボタン3イベント
        mButton3 = (Button) findViewById(R.id.button_3);
        mButton3.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                if(mTimer != null) return;
            	mButtonNo = "3";
            	button_event(mButton3);
            }
        });

        // ボタン4イベント
        //mButton4 = (Button) findViewById(R.id.button_4);
        //mButton4.setOnClickListener(new OnClickListener() {
        //    public void onClick(View v) {
        //		if(mTimer != null) return;
        //    	mButtonNo = "4";
        //    	button_event(mButton4);
        //    }
        //});

        // ボタン5イベント
        //mButton5 = (Button) findViewById(R.id.button_5);
        //mButton5.setOnClickListener(new OnClickListener() {
        //    public void onClick(View v) {
        //		if(mTimer != null) return;
        //    	mButtonNo = "5";
        //    	button_event(mButton5);
        //    }
        //});
        // ボタン6イベント
        mButton6 = (Button) findViewById(R.id.button_6);
        mButton6.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                if(mTimer != null) return;
            	mButtonNo = "6";
            	button_event(mButton6);
            }
        });
        // ボタン7イベント
        //mButton7 = (Button) findViewById(R.id.button_7);
        //mButton7.setOnClickListener(new OnClickListener() {
        //    public void onClick(View v) {
        //		if(mTimer != null) return;
        //    	mButtonNo = "7";
        //    	button_event(mButton7);
        //    }
        //});
        // ボタン8イベント
        mButton8 = (Button) findViewById(R.id.button_8);
        mButton8.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                if(mTimer != null) return;
            	mButtonNo = "8";
            	button_event(mButton8);
            }
        });
        // ボタン9イベント
        mButton9 = (ImageButton) findViewById(R.id.button_9);
        mButton9.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                if(mTimer != null) return;
            	mButtonNo = "9";
            	button_event2(mButton9);
            }
        });

        // ボタン10イベント
        mButton10 = (ImageButton) findViewById(R.id.button_10);
        mButton10.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                if(mTimer != null) return;
            	mButtonNo = "10";
            	button_event2(mButton10);
            }
        });

        // ボタン11イベント
        mButton11 = (ImageButton) findViewById(R.id.button_11);
        mButton11.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                if(mTimer != null) return;
            	mButtonNo = "11";
            	button_event2(mButton11);
            }
        });

        // ボタン12イベント
        mButton12 = (ImageButton) findViewById(R.id.button_12);
        mButton12.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                if(mTimer != null) return;
            	mButtonNo = "12";
            	button_event2(mButton12);
            }
        });

        // ボタン13イベント
        mButton13 = (Button) findViewById(R.id.button_13);
        mButton13.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                if(mTimer != null) return;
            	mButtonNo = "13";
            	button_event(mButton13);
            }
        });

        // ボタン14イベント
        mButton14 = (Button) findViewById(R.id.button_14);
        mButton14.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                if(mTimer != null) return;
            	mButtonNo = "14";
            	button_event(mButton14);
            }
        });

        // ボタン15イベント
        mButton15 = (Button) findViewById(R.id.button_15);
        mButton15.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                if(mTimer != null) return;
            	mButtonNo = "15";
            	button_event(mButton15);
            }
        });
        // ボタン16イベント(CH+)
        mButton16 = (Button) findViewById(R.id.button_16);
        mButton16.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                if(mTimer != null) return;
            	mButtonNo = "16";
            	button_event(mButton16);
            }
        });
        // ボタン17イベント(音声＋)
        mButton17 = (Button) findViewById(R.id.button_17);
        mButton17.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                if(mTimer != null) return;
            	mButtonNo = "17";
            	button_event(mButton17);
            }
        });

        // ボタン18イベント
        //mButton18 = (Button) findViewById(R.id.button_18);
        //mButton18.setOnClickListener(new OnClickListener() {
        //    public void onClick(View v) {
        //		if(mTimer != null) return;
        //    	mButtonNo = "18";
        //    	button_event(mButton18);
        //    }
        //});
        // ボタン19イベント
        mButton19 = (ImageButton) findViewById(R.id.button_19);
        mButton19.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                if(mTimer != null) return;
            	mButtonNo = "19";
            	button_event2(mButton19);
            }
        });

        // ボタン20イベント
        //mButton20 = (Button) findViewById(R.id.button_20);
        //mButton20.setOnClickListener(new OnClickListener() {
        //    public void onClick(View v) {
        //		if(mTimer != null) return;
        //    	mButtonNo = "20";
        //    	button_event(mButton20);
        //    }
        //});

        // ボタン21イベント
        mButton21 = (ImageButton) findViewById(R.id.button_21);
        mButton21.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                if(mTimer != null) return;
            	mButtonNo = "21";
            	button_event2(mButton21);
            }
        });

        // ボタン22イベント
        mButton22 = (ImageButton) findViewById(R.id.button_22);
        mButton22.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                if(mTimer != null) return;
            	mButtonNo = "22";
            	button_event2(mButton22);
            }
        });

        // ボタン23イベント
        mButton23 = (ImageButton) findViewById(R.id.button_23);
        mButton23.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                if(mTimer != null) return;
            	mButtonNo = "23";
            	button_event2(mButton23);
            }
        });

        // ボタン24イベント
        mButton24 = (ImageButton) findViewById(R.id.button_24);
        mButton24.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                if(mTimer != null) return;
            	mButtonNo = "24";
            	button_event2(mButton24);
            }
        });

        // ボタン25イベント
        mButton25 = (Button) findViewById(R.id.button_25);
        mButton25.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                if(mTimer != null) return;
            	mButtonNo = "25";
            	button_event(mButton25);
            }
        });
        // ボタン26イベント
        mButton26 = (Button) findViewById(R.id.button_26);
        mButton26.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                if(mTimer != null) return;
            	mButtonNo = "26";
            	button_event(mButton26);
            }
        });
        // ボタン27イベント
        mButton27 = (Button) findViewById(R.id.button_27);
        mButton27.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                if(mTimer != null) return;
            	mButtonNo = "27";
            	button_event(mButton27);
            }
        });
        // ボタン28イベント(CH-)
        mButton28 = (Button) findViewById(R.id.button_28);
        mButton28.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                if(mTimer != null) return;
            	mButtonNo = "28";
            	button_event(mButton28);
            }
        });
        // ボタン29イベント(音声－)
        mButton29 = (Button) findViewById(R.id.button_29);
        mButton29.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                if(mTimer != null) return;
            	mButtonNo = "29";
            	button_event(mButton29);
            }
        });

        // ボタン30イベント
        mButton30 = (ImageButton) findViewById(R.id.button_30);
        mButton30.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                if(mTimer != null) return;
            	mButtonNo = "30";
            	button_event2(mButton30);
            }
        });

        // ボタン31イベント
        mButton31 = (Button) findViewById(R.id.button_31);
        mButton31.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                if(mTimer != null) return;
            	mButtonNo = "31";
            	button_event(mButton31);
            }
        });

        // ボタン32イベント
        mButton32 = (ImageButton) findViewById(R.id.button_32);
        mButton32.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                if(mTimer != null) return;
            	mButtonNo = "32";
            	button_event2(mButton32);
            }
        });

        // ボタン33イベント
        mButton33 = (ImageButton) findViewById(R.id.button_33);
        mButton33.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                if(mTimer != null) return;
            	mButtonNo = "33";
            	button_event2(mButton33);
            }
        });

        // ボタン34イベント
        mButton34 = (ImageButton) findViewById(R.id.button_34);
        mButton34.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                if(mTimer != null) return;
            	mButtonNo = "34";
            	button_event2(mButton34);
            }
        });

        // ボタン35イベント
        //mButton35 = (Button) findViewById(R.id.button_35);
        //mButton35.setOnClickListener(new OnClickListener() {
        //    public void onClick(View v) {
        //		if(mTimer != null) return;
        //    	mButtonNo = "35";
        //    	button_event(mButton35);
        //    }
        //});
        // ボタン36イベント
        mButton36 = (ImageButton) findViewById(R.id.button_36);
        mButton36.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                if(mTimer != null) return;
            	mButtonNo = "36";
            	button_event2(mButton36);
            }
        });
        // ボタン37イベント
        mButton37 = (Button) findViewById(R.id.button_37);
        mButton37.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                if(mTimer != null) return;
            	mButtonNo = "37";
            	button_event(mButton37);
            }
        });
        // ボタン38イベント
        mButton38 = (Button) findViewById(R.id.button_38);
        mButton38.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                if(mTimer != null) return;
            	mButtonNo = "38";
            	button_event(mButton38);
            }
        });
        // ボタン39イベント
        mButton39 = (Button) findViewById(R.id.button_39);
        mButton39.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                if(mTimer != null) return;
            	mButtonNo = "39";
            	button_event(mButton39);
            }
        });

        // ボタン40イベント
        //mButton40 = (Button) findViewById(R.id.button_40);
        //mButton40.setOnClickListener(new OnClickListener() {
        //    public void onClick(View v) {
        //		if(mTimer != null) return;
        //    	mButtonNo = "40";
        //    	button_event(mButton40);
        //    }
        //});

        // ボタン41イベント
        //mButton41 = (Button) findViewById(R.id.button_41);
        //mButton41.setOnClickListener(new OnClickListener() {
        //    public void onClick(View v) {
        //		if(mTimer != null) return;
        //    	mButtonNo = "41";
        //    	button_event(mButton41);
        //    }
        //});

        // ボタン42イベント
        //mButton42 = (Button) findViewById(R.id.button_42);
        //mButton42.setOnClickListener(new OnClickListener() {
        //    public void onClick(View v) {
        //		if(mTimer != null) return;
        //    	mButtonNo = "42";
        //    	button_event(mButton42);
        //    }
        //});

        // ボタン43イベント
        mButton43 = (ImageButton) findViewById(R.id.button_43);
        mButton43.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                if(mTimer != null) return;
            	mButtonNo = "43";
            	button_event2(mButton43);
            }
        });

        // ボタン44イベント
        //mButton44 = (Button) findViewById(R.id.button_44);
        //mButton44.setOnClickListener(new OnClickListener() {
        //    public void onClick(View v) {
        //		if(mTimer != null) return;
        //    	mButtonNo = "44";
        //    	button_event(mButton44);
        //    }
        //});

        // ボタン45イベント
        mButton45 = (ImageButton) findViewById(R.id.button_45);
        mButton45.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                if(mTimer != null) return;
            	mButtonNo = "45";
            	button_event2(mButton45);
            }
        });
        // ボタン46イベント
        mButton46 = (ImageButton) findViewById(R.id.button_46);
        mButton46.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                if(mTimer != null) return;
            	mButtonNo = "46";
            	button_event2(mButton46);
            }
        });
        // ボタン47イベント
        //mButton47 = (Button) findViewById(R.id.button_47);
        //mButton47.setOnClickListener(new OnClickListener() {
        //    public void onClick(View v) {
        //		if(mTimer != null) return;
        //    	mButtonNo = "47";
        //    	button_event(mButton47);
        //    }
        //});
        // ボタン48イベント
        mButton48 = (ImageButton) findViewById(R.id.button_48);
        mButton48.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                if(mTimer != null) return;
            	mButtonNo = "48";
            	button_event2(mButton48);
            }
        });
        // ボタン49イベント
        mButton49 = (Button) findViewById(R.id.button_49);
        mButton49.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                if(mTimer != null) return;
            	mButtonNo = "49";
            	button_event(mButton49);
            }
        });

        // ボタン50イベント
        mButton50 = (Button) findViewById(R.id.button_50);
        mButton50.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                if(mTimer != null) return;
            	mButtonNo = "50";
            	button_event(mButton50);
            }
        });

        // ボタン51イベント
        mButton51 = (Button) findViewById(R.id.button_51);
        mButton51.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                if(mTimer != null) return;
            	mButtonNo = "51";
            	button_event(mButton51);
            }
        });

        // ボタン52イベント
        mButton52 = (Button) findViewById(R.id.button_52);
        mButton52.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                if(mTimer != null) return;
            	mButtonNo = "52";
            	button_event(mButton52);
            }
        });

        // ボタン53イベント
        mButton53 = (Button) findViewById(R.id.button_53);
        mButton53.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                if(mTimer != null) return;
            	mButtonNo = "53";
            	button_event(mButton53);
            }
        });

        // ボタン54イベント
        mButton54 = (Button) findViewById(R.id.button_54);
        mButton54.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                if(mTimer != null) return;
            	mButtonNo = "54";
            	button_event(mButton54);
            }
        });

        // ボタン55イベント
        //mButton55 = (Button) findViewById(R.id.button_55);
        //mButton55.setOnClickListener(new OnClickListener() {
        //    public void onClick(View v) {
        //		if(mTimer != null) return;
        //    	mButtonNo = "55";
        //    	button_event(mButton55);
        //    }
        //});
        // ボタン56イベント
        mButton56 = (Button) findViewById(R.id.button_56);
        mButton56.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                if(mTimer != null) return;
            	mButtonNo = "56";
            	button_event(mButton56);
            }
        });
        // ボタン57イベント
        mButton57 = (Button) findViewById(R.id.button_57);
        mButton57.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
        		if(mTimer != null) return;
            	mButtonNo = "57";
            	button_event(mButton57);
            }
        });
        // ボタン58イベント
        mButton58 = (Button) findViewById(R.id.button_58);
        mButton58.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
        		if(mTimer != null) return;
            	mButtonNo = "58";
            	button_event(mButton58);
            }
        });
        // ボタン59イベント
        mButton59 = (Button) findViewById(R.id.button_59);
        mButton59.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
        		if(mTimer != null) return;
            	mButtonNo = "59";
            	button_event(mButton59);
            }
        });

        // ボタン60イベント
        mButton60 = (ImageButton) findViewById(R.id.button_60);
        mButton60.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                if(mTimer != null) return;
            	mButtonNo = "60";
            	button_event2(mButton60);
            }
        });

        // ボタンXXXイベント
        //mButtonXXX = (Button) findViewById(R.id.button_XXX);
        //mButtonXXX.setOnClickListener(new OnClickListener() {
        //    public void onClick(View v) {
        //		if(mTimer != null) return;
    	//		mButtonXXX.setEnabled(false);
        //		if(D){Log.d(TAG, "mButtonXXX OnClick() --- mButtonXXX is Disabled.");}
        //    	mButtonNo = "XXX";
        //    	button_event();
    	//		if(mTimer == null){
    	//			//タイマーの初期化処理
    	//			timerTask = new ButtonTimerTask();
    	//			mTimer = new Timer(true);
    	//			mTimer.schedule( timerTask, INHIBIT_TIME);
        //    		if(D){Log.d(TAG, "mButtonXXX OnClick() --- mButtonXXX Timer is Enabled.");}
    	//		}
        //    }

        // Initialize the BluetoothChatService to perform bluetooth connections
        mChatService = new RemoteControlService(this, mHandler);

        // Initialize the buffer for outgoing messages
        mOutStringBuffer = new StringBuffer("");

    }

    public void button_event(Button inObj) {
    	inObj.setEnabled(false);  //無効にする
        if(D){Log.d(TAG, "button_event() --- " + mButtonNo + " is Disabled.");}
        if(mModeFlag==1){
        	// 学習ボタンをクリックしたときの振る舞い SYSTEC
            // コマンド送信後→受信待ち処理→スレッドで待ち？？？
            // コマンド送信(R)
        	mIRSendFlag = 1;
            sendMessage(RCVE_CMD.getBytes());
            Arrays.fill(mCommandBuffer, (byte)0);
        }
        else{
            // 赤外線送信ボタンをクリックしたときの振る舞い SYSTEC
        	mIRSendFlag = 2;
        	if(mExternalMedia == 1){
                // 外部ストレージから読み出して送信
                if(D) Log.d(TAG, "IR Data From External Strage --- Begin.");
                byte[] sendIRCMD = new byte[COMMAND_SIZE+1];
                System.arraycopy(SEND_CMD.getBytes(), 0, sendIRCMD, 0, 1);
                System.arraycopy(extStrage2data(FILE_NAME_STRAGE + mButtonNo, inObj), 0, sendIRCMD, 1, COMMAND_SIZE);
                if(D) Log.d(TAG, "IR Data From External Strage --- End.");
                // 送信
            	sendMessage(sendIRCMD); // SEND_CMD+extStrage2data(FILE_NAME_STRAGE + mButtonNo)
        	}else{
                // 内部ストレージから読み出して送信
                if(D) Log.d(TAG, "IR Data From Internal Strage --- Begin.");
                byte[] sendIRCMD = new byte[COMMAND_SIZE+1];
                System.arraycopy(SEND_CMD.getBytes(), 0, sendIRCMD, 0, 1);
                System.arraycopy(intStrage2Data(FILE_NAME + mButtonNo, inObj), 0, sendIRCMD, 1, COMMAND_SIZE);
                if(D) Log.d(TAG, "IR Data From Internal Strage --- End.");
                // 送信
            	sendMessage(sendIRCMD); // SEND_CMD+intStrage2data(FILE_NAME)
        	}
        }
        // ダブルタッチの不具合解消
    	if(mTimer == null){
    		//タイマーの初期化処理
    		timerTask = new ButtonTimerTask();
    		mTimer = new Timer(true);
    		mTimer.schedule( timerTask, INHIBIT_TIME);
            if(D){Log.d(TAG, "button_event() --- Timer is Enabled.");}
    	}
    }

    // イメージボタン用イベント
    public void button_event2(ImageButton inObj) {
    	inObj.setEnabled(false);  //無効にする
        if(D){Log.d(TAG, "button_event2() --- " + mButtonNo + " is Disabled.");}
        if(mModeFlag==1){
        	// 学習ボタンをクリックしたときの振る舞い SYSTEC
            // コマンド送信後→受信待ち処理→スレッドで待ち？？？
            // コマンド送信(R)
        	mIRSendFlag = 1;
            sendMessage(RCVE_CMD.getBytes());
            Arrays.fill(mCommandBuffer, (byte)0);
        }
        else{
            // 赤外線送信ボタンをクリックしたときの振る舞い SYSTEC
        	mIRSendFlag = 2;
        	if(mExternalMedia == 1){
                // 外部ストレージから読み出して送信
                if(D) Log.d(TAG, "IR Data From External Strage --- Begin.");
                byte[] sendIRCMD = new byte[COMMAND_SIZE+1];
                System.arraycopy(SEND_CMD.getBytes(), 0, sendIRCMD, 0, 1);
                System.arraycopy(extStrage2data2(FILE_NAME_STRAGE + mButtonNo, inObj), 0, sendIRCMD, 1, COMMAND_SIZE);
                if(D) Log.d(TAG, "IR Data From External Strage --- End.");
                // 送信
            	sendMessage(sendIRCMD); // SEND_CMD+extStrage2data(FILE_NAME_STRAGE + mButtonNo)
        	}else{
                // 内部ストレージから読み出して送信
                if(D) Log.d(TAG, "IR Data From Internal Strage --- Begin.");
                byte[] sendIRCMD = new byte[COMMAND_SIZE+1];
                System.arraycopy(SEND_CMD.getBytes(), 0, sendIRCMD, 0, 1);
                System.arraycopy(intStrage2Data2(FILE_NAME + mButtonNo, inObj), 0, sendIRCMD, 1, COMMAND_SIZE);
                if(D) Log.d(TAG, "IR Data From Internal Strage --- End.");
                // 送信
            	sendMessage(sendIRCMD); // SEND_CMD+intStrage2data(FILE_NAME)
        	}
        }
        // ダブルタッチの不具合解消
    	if(mTimer == null){
    		//タイマーの初期化処理
    		timerTask = new ButtonTimerTask();
    		mTimer = new Timer(true);
    		mTimer.schedule( timerTask, INHIBIT_TIME);
            if(D){Log.d(TAG, "button_event2() --- Timer is Enabled.");}
    	}
    }


    @Override
    public synchronized void onPause() {
        super.onPause();
        if(D) Log.e(TAG, "- ON PAUSE -");
    }

    @Override
    public void onStop() {
        super.onStop();
        if(D) Log.e(TAG, "-- ON STOP --");
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        if(CMD_ON_FINIFH){
            // "E"コマンド送信
            sendMessage(FINISH_CMD.getBytes());
        }
        Arrays.fill(mCommandBuffer, (byte)0);
        // Stop the Bluetooth chat services
        if (mChatService != null) mChatService.stop();
        if(D) Log.e(TAG, "--- ON DESTROY ---");
    }


    /**
     * Sends a message.
     * @param message  A string of text to send.
     */
    private void sendMessage(byte[] message) {
        // Check that we're actually connected before trying anything
        if (mChatService.getState() != RemoteControlService.STATE_CONNECTED) {
            Toast.makeText(this, R.string.not_connected, Toast.LENGTH_SHORT).show();
            return;
        }

        // Check that there's actually something to send
        if (message.length > 0) {
            // Get the message bytes and tell the BluetoothChatService to write

            mChatService.write(message);

            // Reset out string buffer to zero and clear the edit text field
            mOutStringBuffer.setLength(0);
        }
    }


    // The Handler that gets information back from the BluetoothChatService
    private final Handler mHandler = new Handler() {
        @Override
        public void handleMessage(Message msg) {
            switch (msg.what) {
            case MESSAGE_STATE_CHANGE:
                if(D) Log.i(TAG, "MESSAGE_STATE_CHANGE: " + msg.arg1);
                switch (msg.arg1) {
                case RemoteControlService.STATE_CONNECTED:
                    mTitle.setText(R.string.title_connected_to);
                    mTitle.append(mConnectedDeviceName);
                    mConversationArrayAdapter.clear();
                    break;
                case RemoteControlService.STATE_CONNECTING:
                    mTitle.setText(R.string.title_connecting);
                    break;
                case RemoteControlService.STATE_LISTEN:
                case RemoteControlService.STATE_NONE:
                    mTitle.setText(R.string.title_not_connected);
                    break;
                }
                break;
            case MESSAGE_WRITE:
            	// スレッド送信イベント
                byte[] writeBuf = (byte[]) msg.obj;
                // construct a string from the buffer
                String writeMessage = new String(writeBuf);
                String binWriteString = "";
                int    aCount;
                int    aInitCount;
                if(mIRSendFlag == 1 || mIRSendFlag == 2){
                	// 赤外線学習
                	aInitCount = 1;
                }else{
                	aInitCount = 0;
                }
                // HEX DUMP SYSTEC
                for(aCount = aInitCount; aCount < writeBuf.length; aCount++){
                	if((aCount - aInitCount)%16 == 0){
                		binWriteString += String.format("\n%02X ", writeBuf[aCount]);
                	}else{
                		binWriteString += String.format("%02X ", writeBuf[aCount]);
                	}
                }
                if(DWIN){
                	if(HANDLED_BINARY){
                		if(mIRSendFlag == 1){
                			mConversationArrayAdapter.add(Build.MODEL+":  " + RCVE_CMD + binWriteString); // Me:
                			//if(D) Log.d(TAG, "書き込み(送信)" + RCVE_CMD + binWriteString + "---バイト数:" + String.valueOf(writeBuf.length));
                		}else if(mIRSendFlag == 2){
                			mConversationArrayAdapter.add(Build.MODEL+":  " + SEND_CMD + "\n"  + binWriteString); // Me:
                			//if(D) Log.d(TAG, "書き込み(送信)" + SEND_CMD  + binWriteString + "---バイト数:" + String.valueOf(writeBuf.length));
                		}else{
                			mConversationArrayAdapter.add(Build.MODEL+":  " + binWriteString); // Me:
                			//if(D) Log.d(TAG, "書き込み(送信)" + binWriteString + "---バイト数:" + String.valueOf(writeBuf.length-1));
                		}
                	}else{
                		if(mIRSendFlag == 1){
                			mConversationArrayAdapter.add(Build.MODEL+":  " + writeMessage); // Me:
                			//if(D) Log.d(TAG, "書き込み(送信)" + writeMessage + "---文字数:" + String.valueOf(binWriteString.length()));
                		}else if(mIRSendFlag == 2){
                			mConversationArrayAdapter.add(Build.MODEL+":  " + writeMessage); // Me:
                			//if(D) Log.d(TAG, "書き込み(送信)" + writeMessage + "---文字数:" + String.valueOf(writeMessage.length()));
                		}else{
                			mConversationArrayAdapter.add(Build.MODEL+":  " + writeMessage); // Me:
                			//if(D) Log.d(TAG, "書き込み(送信)" + writeMessage + "---文字数:" + String.valueOf(writeMessage.length()));
                		}
                	}
                }
                break;
            case MESSAGE_READ:
            	// スレッドからの読み込みイベント
                byte[] readBuf = (byte[]) msg.obj;
                // construct a string from the valid bytes in the buffer
                String readMessage = new String(readBuf, 0, msg.arg1);
                int    aCount1;
                String binReadString = ""; // For HEX DUMP
                // 受信データ
                for(aCount1 = 0 ; aCount1 < msg.arg1 ; aCount1++){
                	binReadString = binReadString.concat(String.format("%1$02X ", readBuf[aCount1]));
                    // コマンドデータを格納　SYSTEC
                	mCommandBuffer[aCount1] = readBuf[aCount1];
                }
                // ListViewに表示 SYSTEC
                if(DWIN){
                	if(HANDLED_BINARY){
                		// バイナリーデータが含まれている場合16進数で表示
                		mConversationArrayAdapter.add(mConnectedDeviceName+":  " + binReadString);
                		mConversationArrayAdapter.add(mConnectedDeviceName + ": 受信バイト数=[" + msg.arg1 + "]");
                	}else{
                		// ASCIIのみの場合そのまま表示
                		mConversationArrayAdapter.add(mConnectedDeviceName+":  " + readMessage);
                		mConversationArrayAdapter.add(mConnectedDeviceName + ": 受信文字数=[" + readMessage.length() + "]");
                	}
                }
                //if(D) Log.d(TAG, "受信データ(画面用HEX表示)=[" + binReadString + "]  受信バイト数:" + String.valueOf(aCount1));

                // 外部ストレージが使えるかどうかで保存先を変える SYSTEC
                if(mExternalMedia == 1){
                    // 外部ストレージに保存 SYSTEC
                    if(D) Log.d(TAG, "External Strage is Mounted. Now Save External.");
                   	data2ExtStrage(mCommandBuffer);
                }else{
                    // 内部ストレージに保存 SYSTEC
                   	if(D) Log.d(TAG, "External Strage is Disabled. Now Save Internal.");
                   	// String readCommands = new String(mCommandBuffer, 0, COMMAND_SIZE);
                   	data2IntStrage(readBuf);
                }
                //ｍbufCount が利設置されるのは "R"コマンドが実行されたとき
                break;
            case MESSAGE_DEVICE_NAME:
                // save the connected device's name
                mConnectedDeviceName = msg.getData().getString(DEVICE_NAME);
                Toast.makeText(getApplicationContext(), mConnectedDeviceName
                         + " に接続しました ", Toast.LENGTH_SHORT).show();
                break;
            case MESSAGE_TOAST:
                Toast.makeText(getApplicationContext(), msg.getData().getString(TOAST),
                               Toast.LENGTH_SHORT).show();
                break;
            }
        }
    };

    // 外部ストレージが利用可能かどうか
    private boolean isExternalStorageAvailable(){
    	boolean state = false;
    	String extStorageState = Environment.getExternalStorageState();
    	Log.i(TAG, "External Strage:"+extStorageState);
    	if (Environment.MEDIA_MOUNTED.equals(extStorageState)){
    		state = true;
    	}
    	return state;
    }

    // バイトデータ→外部ストレージ保存
    private void data2ExtStrage(byte[] inData){
    	// FileReadWrite
    	if(D) Log.d(TAG, "Write External Strage Dir is ["+mExternalDirectory+"]");
    	try{
        	FileReadWrite.writeFile(mExternalDirectory+"/"+FILE_NAME_STRAGE + mButtonNo, inData);
    	}catch(IOException e){
        	// if(D) Log.d(TAG, "Write External Strage Error: "+e.toString());
            showDialog(this, "",  "学習させることができませんでした。\n(ファイル書き込み失敗)");
    	}
    }
    // バイトデータ->ファイル　ラッパー
    private void data2IntStrage(byte[] message){
        try{
            data2file(this, message, FILE_NAME + mButtonNo);
        }catch(IOException e){
            showDialog(this, "",  "学習させることができませんでした。\n(ファイル書き込み失敗)");
        }
    }

    // バイトデータ->内部ストレージ保存
    private void data2file(Context context, byte[] w, String filename) throws IOException{
    	if(D) Log.d(TAG, "Receive Data To Internal Strage File: Write Operation");
    	OutputStream out=null;
    	try{
    		// Stream Open
    		out = context.openFileOutput(filename, Context.MODE_PRIVATE);
    		// 書き込み
    		out.write(w,0,w.length);
    		// Close
    		out.close();
    	} catch(IOException e){
    		try{
    			if(out != null) out.close();
    		} catch(IOException e2){}
    		throw e;
    	}
    }

    // 外部ストレージ読み出し→バイトデータ
    private byte[] extStrage2data(String filename, Button inObj){
    	if(D) Log.d(TAG, "Read from External Strage Dir is ["+mExternalDirectory+"]");
    	byte[] retData = new byte[COMMAND_SIZE];
    	try{
    		byte[] readData = FileReadWrite.readFile(mExternalDirectory+"/"+filename);
        	if(D2){
            	int    aCount;
        		String aDebugData="";
        		for(aCount = 0; aCount < readData.length; aCount++){
        			aDebugData += String.format("%02X ", readData[aCount]);
        		}
        		Log.d(TAG, "Read From External Strage Data: "+ aDebugData + "[" + String.valueOf(readData.length) + "]");
        	}
        	System.arraycopy(readData, 0, retData, 0, COMMAND_SIZE);
    	}catch(IOException e){
            showDialog(this, "",  "学習していないボタンがタップされました。");
            inObj.setEnabled(true);
        	// if(D) Log.e(TAG, "Read From External Strage Error: "+e.toString());
    	}
    	return retData;
    }

    private byte[] extStrage2data2(String filename, ImageButton inObj){
    	if(D) Log.d(TAG, "Read from External Strage Dir is ["+mExternalDirectory+"]");
    	byte[] retData = new byte[COMMAND_SIZE];
    	try{
    		byte[] readData = FileReadWrite.readFile(mExternalDirectory+"/"+filename);
        	if(D2){
            	int    aCount;
        		String aDebugData="";
        		for(aCount = 0; aCount < readData.length; aCount++){
        			aDebugData += String.format("%02X ", readData[aCount]);
        		}
        		Log.d(TAG, "Read From External Strage Data: "+ aDebugData + "[" + String.valueOf(readData.length) + "]");
        	}
        	System.arraycopy(readData, 0, retData, 0, COMMAND_SIZE);
    	}catch(IOException e){
            showDialog(this, "",  "学習していないボタンがタップされました。");
            inObj.setEnabled(true);
        	// if(D) Log.e(TAG, "Read From External Strage Error: "+e.toString());
    	}
    	return retData;
    }


    // 内部ストレージ読み出し→バイトデータラッパー SYSTEC
    private byte[] intStrage2Data(String filename, Button inObj){
    	byte[] retData = new byte[COMMAND_SIZE];
    	try{
		    byte[] readMessage = file2data(this, FILE_NAME + mButtonNo);
		    if(D2){
		    	int    aCount;
		    	String aDebugData="";
		    	for(aCount = 0; aCount < readMessage.length; aCount++){
		    		aDebugData += String.format("%02X ", readMessage[aCount]);
		    	}
		    	Log.d(TAG, "Read From Internal Strage Data: "+ aDebugData + "[" + String.valueOf(readMessage.length) + "]");
		    }
        	System.arraycopy(readMessage, 0, retData, 0, COMMAND_SIZE);
		}catch(IOException e){
            showDialog(this, "",  "学習していないボタンがタップされました。");
             inObj.setEnabled(true);
		}
		return retData;
    }


    private byte[] intStrage2Data2(String filename, ImageButton inObj){
    	byte[] retData = new byte[COMMAND_SIZE];
    	try{
		    byte[] readMessage = file2data(this, FILE_NAME + mButtonNo);
		    if(D2){
		    	int    aCount;
		    	String aDebugData="";
		    	for(aCount = 0; aCount < readMessage.length; aCount++){
		    		aDebugData += String.format("%02X ", readMessage[aCount]);
		    	}
		    	Log.d(TAG, "Read From Internal Strage Data: "+ aDebugData + "[" + String.valueOf(readMessage.length) + "]");
		    }
        	System.arraycopy(readMessage, 0, retData, 0, COMMAND_SIZE);
		}catch(IOException e){
             showDialog(this, "",  "学習していないボタンがタップされました。");
             inObj.setEnabled(true);
		}
		return retData;
    }


    // 内部ストレージ読み出し→バイトデータ SYSTEC
    private byte[] file2data(Context context, String fileName) throws IOException {
        if(D) Log.d(TAG, "Read Transmit Data From File Operation");
        int size;
        byte[] w=new byte[BUFFER_SIZE];
        FileInputStream in=null;
        ByteArrayOutputStream out=null;
        try {
            //ファイル入力ストリームのオープン
            in=context.openFileInput(fileName);

            //バイト配列の読み込み
            out=new ByteArrayOutputStream();
            while (true) {
                size=in.read(w,0,BUFFER_SIZE);
                if (size<=0) break;
                out.write(w,0,size);
            }
            out.close();

            //ファイル入力ストリームのクローズ
            in.close();

            //ByteArrayOutputStreamオブジェクトのバイト配列化
            return out.toByteArray();
        } catch (IOException e) {
            if(D) Log.d(TAG, "Read Tansmit Data From File Error"+e.toString());
            try {
                if (in!=null) in.close();
                if (out!=null) out.close();
            } catch (Exception e2) {
            }
            throw e;
        }
    }

    //ダイアログの表示 SYSTEC
    private void showDialog(Context context,String title,String text) {
        AlertDialog.Builder ad=new AlertDialog.Builder(context);
        ad.setTitle(title);
        ad.setMessage(text);
        ad.setPositiveButton("OK",null);
        ad.show();
    }

    // ダイアログの表示(はい・いいえ) SYSTEC
    private void showIndicadeDialog(Context context,String title,String text) {
        AlertDialog.Builder ad=new AlertDialog.Builder(context);
        ad.setTitle(title);
        ad.setMessage(text);
        ad.setPositiveButton("OK", new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int which) {
				// デフォルト接続開始 SYSTEC
		        if(D) Log.d(TAG, "showIndicadeDialog which=[" + Integer.toString(which) + "]");
				mBluetoothAdapter.cancelDiscovery();
				connectExistingDevice(mMainConnectDeviceAddress, false); // 安全でない接続
			}
		});
        ad.show();
    }

    // ダイアログの表示(OK)
    private void showMsgDialog(Context context,String title,String text) {
        AlertDialog.Builder msg=new AlertDialog.Builder(context);
        msg.setTitle(title);
        msg.setMessage(text);
        msg.setPositiveButton("OK", null);
        msg.show();
    }


    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        if(D) Log.d(TAG, "onActivityResult ResultCode=[" + Integer.toString(resultCode) + "]");
        switch (requestCode) {
        case REQUEST_CONNECT_DEVICE_SECURE:
            // When DeviceListActivity returns with a device to connect
            if (resultCode == Activity.RESULT_OK) {
                connectDevice(data, true);
            }
            break;
        case REQUEST_CONNECT_DEVICE_INSECURE:
            // When DeviceListActivity returns with a device to connect
            if (resultCode == Activity.RESULT_OK) {
                connectDevice(data, false);
            }
            break;
        case REQUEST_ENABLE_BT:
            // When the request to enable Bluetooth returns
            if (resultCode == Activity.RESULT_OK) {
                // Bluetooth is now enabled, so set up a chat session
                setupChat();
            } else {
                // User did not enable Bluetooth or an error occured
                Log.d(TAG, "BT not enabled");
                Toast.makeText(this, R.string.bt_not_enabled_leaving, Toast.LENGTH_SHORT).show();
                finish();
            }
        case RESULT_OK:
            if(D) Log.i(TAG, "onActivityResult Result OK");
        }
    }
    // DeviceListActivityからのパラメータを元に接続 SYSTEC
    private void connectDevice(Intent data, boolean secure) {
        // Get the device MAC address
        String address = data.getExtras().getString(DeviceListActivity.EXTRA_DEVICE_ADDRESS);
        Log.i(TAG, "Connect Remote Address=["+address+"]");
        // Get the BLuetoothDevice object
        BluetoothDevice device = mBluetoothAdapter.getRemoteDevice(address);
        // デバグ
        if(D){Log.d(TAG, "ConnectThread(): Remote Class=[" + Integer.toHexString(device.getBluetoothClass().getDeviceClass()) + "]");}
        // Attempt to connect to the device
        mChatService.connect(device, secure);
    }
    // 既存のデバイスに接続 SYSTEC
    private void connectExistingDevice(String address, boolean secure){
    	//
        BluetoothDevice device = mBluetoothAdapter.getRemoteDevice(address);
        if(D){Log.d(TAG, "connectExistingDevice(): Remote Class=[" + Integer.toHexString(device.getBluetoothClass().getDeviceClass()) + "]");}
        // Attempt to connect to the device
        mChatService.connect(device, secure);
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.option_menu, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
//        Intent serverIntent = null;
        switch (item.getItemId()) {
        //モード切替
        case R.id.mode_change:
            // Ensure this device is discoverable by others
        	TableLayout mainTableLayout = (TableLayout)findViewById(R.id.maintbl);
            if(mModeFlag==1){
            	mModeFlag=0;
            	mainTableLayout.setBackgroundColor(Color.TRANSPARENT);
            	showDialog(this,"モード切替","通常モードになりました");
            }else{
            	mModeFlag=1;
            	// mainTableLayout.setBackgroundColor(R.color.aqua);  // リソースの値を使用する場合はこちらを有効
            	mainTableLayout.setBackgroundColor(Color.argb(192, 255, 255, 0));
            	showDialog(this,"モード切替","学習モードになりました");
            }
            return true;
        //終了
        case R.id.terminate:
            // Finish APP
            finish();
            return true;
        }
        return false;
    }

}
